@@ -1,6 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const GConf = imports.gi.GConf;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Pango = imports.gi.Pango;
@@ -223,7 +224,11 @@ WindowList.prototype = {
         let tracker = Shell.WindowTracker.get_default();
         for (let i = 0; i < windows.length; i++) {
             let metaWindow = windows[i].metaWindow;
-            metaWindow.connect('unmanaged', Lang.bind(this, this._updateWindowList));
+            // Avoid multiple connections
+            if (!metaWindow._lookingGlassManaged) {
+                metaWindow.connect('unmanaged', Lang.bind(this, this._updateWindowList));
+                metaWindow._lookingGlassManaged = true;
+            }
             let box = new St.BoxLayout({ vertical: true });
             this.actor.add(box);
             let windowLink = new ObjLink(metaWindow, metaWindow.title);
@@ -398,14 +403,9 @@ Inspector.prototype = {
             let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
                                                        stageX,
                                                        stageY);
-            let id, style_class;
-            if (target instanceof St.Widget) {
-                id = target.get_theme_node().get_element_id();
-                style_class = target.get_theme_node().get_element_class();
-            }
-            let position = '<inspect x: ' + stageX + ' y: ' + stageY + '>';
-            let style = '<style id: ' + id  + ' class: ' + style_class + '>';
-            displayText.text = position + ' ' + style + ' ' + target;
+            let position = '[inspect x: ' + stageX + ' y: ' + stageY + ']';
+            displayText.text = '';
+            displayText.text = position + ' ' + target;
             if (borderPaintTarget != null)
                 borderPaintTarget.disconnect(borderPaintId);
             borderPaintTarget = target;
@@ -427,6 +427,11 @@ ErrorLog.prototype = {
         this.actor = new St.BoxLayout();
         this.text = new St.Label();
         this.actor.add(this.text);
+        // We need to override StLabel's default ellipsization when
+        // using line_wrap; otherwise ClutterText's layout is going
+        // to constrain both the width and height, which prevents
+        // scrolling.
+        this.text.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
         this.text.clutter_text.line_wrap = true;
         this.actor.connect('notify::mapped', Lang.bind(this, this._renderText));
     },
@@ -578,10 +583,10 @@ LookingGlass.prototype = {
                                         vertical: true,
                                         visible: false });
 
-        let gconf = Shell.GConf.get_default();
-        gconf.watch_directory('/desktop/gnome/interface');
-        gconf.connect('changed::/desktop/gnome/interface/monospace_font_name',
-                      Lang.bind(this, this._updateFont));
+        let gconf = GConf.Client.get_default();
+        gconf.add_dir('/desktop/gnome/interface', GConf.ClientPreloadType.PRELOAD_NONE);
+        gconf.notify_add('/desktop/gnome/interface/monospace_font_name',
+                         Lang.bind(this, this._updateFont));
         this._updateFont();
 
         Main.uiGroup.add_actor(this.actor);
@@ -692,7 +697,7 @@ LookingGlass.prototype = {
     },
 
     _updateFont: function() {
-        let gconf = Shell.GConf.get_default();
+        let gconf = GConf.Client.get_default();
         let fontName = gconf.get_string('/desktop/gnome/interface/monospace_font_name');
         // This is mishandled by the scanner - should by Pango.FontDescription_from_string(fontName);
         // https://bugzilla.gnome.org/show_bug.cgi?id=595889
