@@ -14,12 +14,15 @@ const _ = Gettext.gettext;
 
 const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
+const Tweener = imports.ui.tweener;
 
 const MAX_FILE_DELETED_BEFORE_INVALID = 10;
 
-const HISTORY_KEY = 'run_dialog/history';
+const HISTORY_KEY = 'command-history';
 const HISTORY_LIMIT = 512;
 
+const DIALOG_FADE_TIME = 0.1;
+
 function CommandCompleter() {
     this._init();
 }
@@ -176,17 +179,16 @@ RunDialog.prototype = {
     _init : function() {
         this._isOpen = false;
 
-        this._gconf = Shell.GConf.get_default();
-        this._gconf.connect('changed::development_tools', Lang.bind(this, function () {
-            this._enableInternalCommands = this._gconf.get_boolean('development_tools');
+        global.settings.connect('changed::development-tools', Lang.bind(this, function () {
+            this._enableInternalCommands = global.settings.get_boolean('development-tools');
         }));
-        this._enableInternalCommands = this._gconf.get_boolean('development_tools');
+        this._enableInternalCommands = global.settings.get_boolean('development-tools');
 
-        this._history = this._gconf.get_string_list(HISTORY_KEY);
+        this._history = global.settings.get_strv(HISTORY_KEY);
         this._historyIndex = -1;
 
-        this._gconf.connect('changed::' + HISTORY_KEY, Lang.bind(this, function() {
-            this._history = this._gconf.get_string_list(HISTORY_KEY);
+        global.settings.connect('changed::' + HISTORY_KEY, Lang.bind(this, function() {
+            this._history = global.settings.get_strv(HISTORY_KEY);
             this._historyIndex = this._history.length;
         }));
 
@@ -215,13 +217,14 @@ RunDialog.prototype = {
                                           x: 0, y: 0 });
         Main.uiGroup.add_actor(this._group);
 
-        let lightbox = new Lightbox.Lightbox(this._group, true);
+        this._lightbox = new Lightbox.Lightbox(this._group,
+                                               { inhibitEvents: true });
 
         this._box = new St.Bin({ x_align: St.Align.MIDDLE,
                                  y_align: St.Align.MIDDLE });
 
         this._group.add_actor(this._box);
-        lightbox.highlight(this._box);
+        this._lightbox.highlight(this._box);
 
         let dialogBox = new St.BoxLayout({ style_class: 'run-dialog', vertical: true });
 
@@ -323,13 +326,14 @@ RunDialog.prototype = {
         if (this._history.length > HISTORY_LIMIT) {
             this._history.splice(0, this._history.length - HISTORY_LIMIT);
         }
-        this._gconf.set_string_list(HISTORY_KEY, this._history);
+        global.settings.set_strv(HISTORY_KEY, this._history);
     },
 
     _run : function(input, inTerminal) {
         let command = input;
 
-        if (this._history.length > 0 && this._history[this._history.length - 1] != input) {
+        if (this._history.length == 0 ||
+            this._history[this._history.length - 1] != input) {
             this._history.push(input);
             this._saveHistory();
         }
@@ -413,7 +417,14 @@ RunDialog.prototype = {
         this._box.set_size(monitor.width, monitor.height);
 
         this._isOpen = true;
+        this._lightbox.show();
+        this._group.opacity = 0;
         this._group.show();
+        Tweener.addTween(this._group,
+                         { opacity: 255,
+                           time: DIALOG_FADE_TIME,
+                           transition: 'easeOutQuad'
+                         });
 
         global.stage.set_key_focus(this._entryText);
     },
@@ -423,14 +434,20 @@ RunDialog.prototype = {
             return;
 
         this._isOpen = false;
-
-        this._errorBox.hide();
         this._commandError = false;
 
-        this._group.hide();
-        this._entryText.set_text('');
-
         Main.popModal(this._group);
+
+        Tweener.addTween(this._group,
+                         { opacity: 0,
+                           time: DIALOG_FADE_TIME,
+                           transition: 'easeOutQuad',
+                           onComplete: Lang.bind(this, function() {
+                               this._errorBox.hide();
+                               this._group.hide();
+                               this._entryText.set_text('');
+                           })
+                         });
     }
 };
 Signals.addSignalMethods(RunDialog.prototype);
