@@ -9,7 +9,9 @@ const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 
 const GnomeSession = imports.misc.gnomeSession;
+const Main = imports.ui.main;
 const Panel = imports.ui.panel;
+const PopupMenu = imports.ui.popupMenu;
 
 // Adapted from gdm/gui/user-switch-applet/applet.c
 //
@@ -38,11 +40,10 @@ StatusMenuButton.prototype = {
         box.add(this._iconBox, { y_align: St.Align.MIDDLE, y_fill: false });
 
         let textureCache = St.TextureCache.get_default();
-        // FIXME: these icons are all wrong (likewise in createSubMenu)
-        this._availableIcon = textureCache.load_icon_name('gtk-yes', 16);
-        this._busyIcon = textureCache.load_icon_name('gtk-no', 16);
-        this._invisibleIcon = textureCache.load_icon_name('gtk-close', 16);
-        this._idleIcon = textureCache.load_icon_name('gtk-media-pause', 16);
+        this._availableIcon = textureCache.load_icon_name('user-available', 16);
+        this._busyIcon = textureCache.load_icon_name('user-busy', 16);
+        this._invisibleIcon = textureCache.load_icon_name('user-invisible', 16);
+        this._idleIcon = textureCache.load_icon_name('user-idle', 16);
 
         this._presence.connect('StatusChanged', Lang.bind(this, this._updatePresenceIcon));
         this._presence.getStatus(Lang.bind(this, this._updatePresenceIcon));
@@ -87,46 +88,46 @@ StatusMenuButton.prototype = {
     _createSubMenu: function() {
         let item;
 
-        item = new Panel.PanelImageMenuItem(_("Available"), 'gtk-yes', true);
+        item = new PopupMenu.PopupImageMenuItem(_("Available"), 'user-available', true);
         item.connect('activate', Lang.bind(this, this._setPresenceStatus, GnomeSession.PresenceStatus.AVAILABLE));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Busy"), 'gtk-no', true);
+        item = new PopupMenu.PopupImageMenuItem(_("Busy"), 'user-busy', true);
         item.connect('activate', Lang.bind(this, this._setPresenceStatus, GnomeSession.PresenceStatus.BUSY));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Invisible"), 'gtk-close', true);
+        item = new PopupMenu.PopupImageMenuItem(_("Invisible"), 'user-invisible', true);
         item.connect('activate', Lang.bind(this, this._setPresenceStatus, GnomeSession.PresenceStatus.INVISIBLE));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelSeparatorMenuItem();
+        item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Account Information..."), 'user-info');
+        item = new PopupMenu.PopupImageMenuItem(_("Account Information..."), 'user-info');
         item.connect('activate', Lang.bind(this, this._onAccountInformationActivate));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("System Preferences..."), 'preferences-desktop');
+        item = new PopupMenu.PopupImageMenuItem(_("System Preferences..."), 'preferences-desktop');
         item.connect('activate', Lang.bind(this, this._onPreferencesActivate));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelSeparatorMenuItem();
+        item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Lock Screen"), 'system-lock-screen');
+        item = new PopupMenu.PopupImageMenuItem(_("Lock Screen"), 'system-lock-screen');
         item.connect('activate', Lang.bind(this, this._onLockScreenActivate));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Switch User"), 'system-users');
+        item = new PopupMenu.PopupImageMenuItem(_("Switch User"), 'system-users');
         item.connect('activate', Lang.bind(this, this._onLoginScreenActivate));
         this.menu.addMenuItem(item);
         this._loginScreenItem = item;
 
-        item = new Panel.PanelImageMenuItem(_("Log Out..."), 'system-log-out');
+        item = new PopupMenu.PopupImageMenuItem(_("Log Out..."), 'system-log-out');
         item.connect('activate', Lang.bind(this, this._onQuitSessionActivate));
         this.menu.addMenuItem(item);
 
-        item = new Panel.PanelImageMenuItem(_("Shut Down..."), 'system-shutdown');
+        item = new PopupMenu.PopupImageMenuItem(_("Shut Down..."), 'system-shutdown');
         item.connect('activate', Lang.bind(this, this._onShutDownActivate));
         this.menu.addMenuItem(item);
     },
@@ -136,27 +137,33 @@ StatusMenuButton.prototype = {
     },
 
     _onAccountInformationActivate: function() {
+        Main.overview.hide();
         this._spawn(['gnome-about-me']);
     },
 
     _onPreferencesActivate: function() {
+        Main.overview.hide();
         this._spawn(['gnome-control-center']);
     },
 
     _onLockScreenActivate: function() {
+        Main.overview.hide();
         this._spawn(['gnome-screensaver-command', '--lock']);
     },
 
     _onLoginScreenActivate: function() {
+        Main.overview.hide();
         this._gdm.goto_login_session();
         this._onLockScreenActivate();
     },
 
     _onQuitSessionActivate: function() {
+        Main.overview.hide();
         this._spawn(['gnome-session-save', '--logout-dialog']);
     },
 
     _onShutDownActivate: function() {
+        Main.overview.hide();
         this._spawn(['gnome-session-save', '--shutdown-dialog']);
     },
 
