@@ -45,7 +45,6 @@
 #include "st-button.h"
 
 #include "st-marshal.h"
-#include "st-texture-frame.h"
 #include "st-texture-cache.h"
 #include "st-private.h"
 
@@ -55,8 +54,7 @@ enum
 
   PROP_LABEL,
   PROP_TOGGLE_MODE,
-  PROP_CHECKED,
-  PROP_TRANSITION_DURATION
+  PROP_CHECKED
 };
 
 enum
@@ -73,17 +71,11 @@ struct _StButtonPrivate
 {
   gchar            *text;
 
-  guint8            old_opacity;
-
   guint             is_pressed : 1;
   guint             is_hover : 1;
   guint             is_checked : 1;
   guint             is_toggle : 1;
 
-  gint              transition_duration;
-
-  ClutterAnimation *animation;
-
   gint              spacing;
 };
 
@@ -133,21 +125,13 @@ st_button_style_changed (StWidget *widget)
 static void
 st_button_real_pressed (StButton *button)
 {
-  st_widget_set_style_pseudo_class ((StWidget*) button, "active");
+  st_widget_add_style_pseudo_class ((StWidget*) button, "active");
 }
 
 static void
 st_button_real_released (StButton *button)
 {
-  StButtonPrivate *priv = button->priv;
-
-  if (priv->is_checked)
-    st_widget_set_style_pseudo_class ((StWidget*) button, "checked");
-  else if (!priv->is_hover)
-    st_widget_set_style_pseudo_class ((StWidget*) button, NULL);
-  else
-    st_widget_set_style_pseudo_class ((StWidget*) button, "hover");
-
+  st_widget_remove_style_pseudo_class ((StWidget*) button, "active");
 }
 
 static gboolean
@@ -212,8 +196,7 @@ st_button_enter (ClutterActor         *actor,
 {
   StButton *button = ST_BUTTON (actor);
 
-  if (!button->priv->is_checked)
-    st_widget_set_style_pseudo_class ((StWidget*) button, "hover");
+  st_widget_add_style_pseudo_class ((StWidget*) button, "hover");
 
   button->priv->is_hover = 1;
 
@@ -240,10 +223,7 @@ st_button_leave (ClutterActor         *actor,
         klass->released (button);
     }
 
-  if (button->priv->is_checked)
-    st_widget_set_style_pseudo_class ((StWidget*) button, "checked");
-  else
-    st_widget_set_style_pseudo_class ((StWidget*) button, NULL);
+  st_widget_remove_style_pseudo_class ((StWidget*) button, "hover");
 
   return CLUTTER_ACTOR_CLASS (st_button_parent_class)->leave_event (actor, event);
 }
@@ -255,7 +235,6 @@ st_button_set_property (GObject      *gobject,
                         GParamSpec   *pspec)
 {
   StButton *button = ST_BUTTON (gobject);
-  StButtonPrivate *priv = ST_BUTTON (gobject)->priv;
 
   switch (prop_id)
     {
@@ -268,9 +247,6 @@ st_button_set_property (GObject      *gobject,
     case PROP_CHECKED:
       st_button_set_checked (button, g_value_get_boolean (value));
       break;
-    case PROP_TRANSITION_DURATION:
-      priv->transition_duration = g_value_get_int (value);
-      break;
 
 
     default:
@@ -298,9 +274,6 @@ st_button_get_property (GObject    *gobject,
     case PROP_CHECKED:
       g_value_set_boolean (value, priv->is_checked);
       break;
-    case PROP_TRANSITION_DURATION:
-      g_value_set_int (value, priv->transition_duration);
-      break;
 
 
     default:
@@ -362,12 +335,6 @@ st_button_class_init (StButtonClass *klass)
                                 FALSE, G_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHECKED, pspec);
 
-  pspec = g_param_spec_int ("transition-duration",
-                            "Transition Duration",
-                            "Duration of the state transition effect",
-                            0, G_MAXINT, 120, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TRANSITION_DURATION, pspec);
-
 
   /**
    * StButton::clicked:
@@ -391,7 +358,6 @@ static void
 st_button_init (StButton *button)
 {
   button->priv = ST_BUTTON_GET_PRIVATE (button);
-  button->priv->transition_duration = 120;
   button->priv->spacing = 6;
 
   clutter_actor_set_reactive ((ClutterActor *) button, TRUE);
@@ -494,7 +460,7 @@ st_button_set_label (StButton    *button,
  *
  * Get the toggle mode status of the button.
  *
- * Returns: #TRUE if toggle mode is set, otherwise #FALSE
+ * Returns: %TRUE if toggle mode is set, otherwise %FALSE
  */
 gboolean
 st_button_get_toggle_mode (StButton *button)
@@ -507,7 +473,7 @@ st_button_get_toggle_mode (StButton *button)
 /**
  * st_button_set_toggle_mode:
  * @button: a #Stbutton
- * @toggle: #TRUE or #FALSE
+ * @toggle: %TRUE or %FALSE
  *
  * Enables or disables toggle mode for the button. In toggle mode, the active
  * state will be "toggled" when the user clicks the button.
@@ -529,7 +495,7 @@ st_button_set_toggle_mode (StButton *button,
  *
  * Get the state of the button that is in toggle mode.
  *
- * Returns: #TRUE if the button is checked, or #FALSE if not
+ * Returns: %TRUE if the button is checked, or %FALSE if not
  */
 gboolean
 st_button_get_checked (StButton *button)
@@ -542,10 +508,10 @@ st_button_get_checked (StButton *button)
 /**
  * st_button_set_checked:
  * @button: a #Stbutton
- * @checked: #TRUE or #FALSE
+ * @checked: %TRUE or %FALSE
  *
  * Sets the pressed state of the button. This is only really useful if the
- * button has #toggle-mode mode set to #TRUE.
+ * button has #toggle-mode mode set to %TRUE.
  */
 void
 st_button_set_checked (StButton *button,
@@ -558,12 +524,9 @@ st_button_set_checked (StButton *button,
       button->priv->is_checked = checked;
 
       if (checked)
-        st_widget_set_style_pseudo_class ((StWidget*) button, "checked");
+        st_widget_add_style_pseudo_class ((StWidget*) button, "checked");
       else
-      if (button->priv->is_hover)
-        st_widget_set_style_pseudo_class ((StWidget*) button, "hover");
-      else
-        st_widget_set_style_pseudo_class ((StWidget*) button, NULL);
+        st_widget_remove_style_pseudo_class ((StWidget*) button, "checked");
     }
 
   g_object_notify (G_OBJECT (button), "checked");
