@@ -64,8 +64,6 @@ enum
 
 struct _StTablePrivate
 {
-  GSList *children;
-
   gint    col_spacing;
   gint    row_spacing;
 
@@ -89,11 +87,11 @@ struct _StTablePrivate
   guint   homogeneous : 1;
 };
 
-static void st_container_iface_init (ClutterContainerIface *iface);
+static void st_table_container_iface_init (ClutterContainerIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (StTable, st_table, ST_TYPE_WIDGET,
+G_DEFINE_TYPE_WITH_CODE (StTable, st_table, ST_TYPE_CONTAINER,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
-                                                st_container_iface_init));
+                                                st_table_container_iface_init));
 
 
 
@@ -101,95 +99,50 @@ G_DEFINE_TYPE_WITH_CODE (StTable, st_table, ST_TYPE_WIDGET,
  * ClutterContainer Implementation
  */
 static void
-st_container_add_actor (ClutterContainer *container,
+st_table_actor_removed (ClutterContainer *container,
                         ClutterActor     *actor)
 {
   StTablePrivate *priv = ST_TABLE (container)->priv;
+  GList *list, *children;
+  gint n_rows = 0;
+  gint n_cols = 0;
 
-  clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
-
-
-  priv->children = g_slist_append (priv->children, actor);
-
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
-
-  g_signal_emit_by_name (container, "actor-added", actor);
-}
-
-static void
-st_container_remove_actor (ClutterContainer *container,
-                           ClutterActor     *actor)
-{
-  StTablePrivate *priv = ST_TABLE (container)->priv;
-
-  GSList *item = NULL;
-
-  item = g_slist_find (priv->children, actor);
-
-  if (item == NULL)
+  /* Calculate and update the number of rows / columns */
+  children = st_container_get_children_list (ST_CONTAINER (container));
+  for (list = children; list; list = list->next)
     {
-      g_warning ("Widget of type '%s' is not a child of container of type '%s'",
-                 g_type_name (G_OBJECT_TYPE (actor)),
-                 g_type_name (G_OBJECT_TYPE (container)));
-      return;
+      ClutterActor *child = CLUTTER_ACTOR (list->data);
+      StTableChild *meta;
+
+      if (child == actor)
+          continue;
+
+      meta = (StTableChild *) clutter_container_get_child_meta (container, child);
+      n_rows = MAX (n_rows, meta->row + 1);
+      n_cols = MAX (n_cols, meta->col + 1);
     }
 
-  g_object_ref (actor);
+  g_object_freeze_notify (G_OBJECT (container));
 
-  priv->children = g_slist_delete_link (priv->children, item);
-  clutter_actor_unparent (actor);
+  if (priv->n_rows != n_rows)
+    {
+      priv->n_rows = n_rows;
+      g_object_notify (G_OBJECT (container), "row-count");
+    }
 
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
+  if (priv->n_cols != n_cols)
+    {
+      priv->n_cols = n_cols;
+      g_object_notify (G_OBJECT (container), "column-count");
+    }
 
-  g_signal_emit_by_name (container, "actor-removed", actor);
-
-  g_object_unref (actor);
+  g_object_thaw_notify (G_OBJECT (container));
 }
 
 static void
-st_container_foreach (ClutterContainer *container,
-                      ClutterCallback   callback,
-                      gpointer          callback_data)
+st_table_container_iface_init (ClutterContainerIface *iface)
 {
-  StTablePrivate *priv = ST_TABLE (container)->priv;
-
-  g_slist_foreach (priv->children, (GFunc) callback, callback_data);
-}
-
-static void
-st_container_lower (ClutterContainer *container,
-                    ClutterActor     *actor,
-                    ClutterActor     *sibling)
-{
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
-}
-
-static void
-st_container_raise (ClutterContainer *container,
-                    ClutterActor     *actor,
-                    ClutterActor     *sibling)
-{
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
-}
-
-static void
-st_container_sort_depth_order (ClutterContainer *container)
-{
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
-}
-
-static void
-st_container_iface_init (ClutterContainerIface *iface)
-{
-  iface->add = st_container_add_actor;
-  iface->remove = st_container_remove_actor;
-  iface->foreach = st_container_foreach;
-  iface->lower = st_container_lower;
-  iface->raise = st_container_raise;
-  iface->sort_depth_order = st_container_sort_depth_order;
+  iface->actor_removed = st_table_actor_removed;
   iface->child_meta_type = ST_TYPE_TABLE_CHILD;
 }
 
@@ -267,23 +220,12 @@ st_table_finalize (GObject *gobject)
   G_OBJECT_CLASS (st_table_parent_class)->finalize (gobject);
 }
 
-static void
-st_table_dispose (GObject *gobject)
-{
-  StTablePrivate *priv = ST_TABLE (gobject)->priv;
-
-  while (priv->children)
-    clutter_actor_destroy (priv->children->data);
-
-  G_OBJECT_CLASS (st_table_parent_class)->dispose (gobject);
-}
-
 static void
 st_table_homogeneous_allocate (ClutterActor          *self,
                                const ClutterActorBox *content_box,
                                gboolean               flags)
 {
-  GSList *list;
+  GList *list, *children;
   gfloat col_width, row_height;
   gint row_spacing, col_spacing;
   StTablePrivate *priv = ST_TABLE (self)->priv;
@@ -292,14 +234,15 @@ st_table_homogeneous_allocate (ClutterActor          *self,
   col_spacing = priv->col_spacing;
   row_spacing = priv->row_spacing;
 
-  col_width = (content_box->x2 - content_box->x1
-               - (col_spacing * (priv->n_cols - 1)))
-              / priv->n_cols;
-  row_height = (content_box->y2 - content_box->y1
-                - (row_spacing * (priv->n_rows - 1)))
-               / priv->n_rows;
+  col_width = (int) ((content_box->x2 - content_box->x1
+                      - (col_spacing * (priv->n_cols - 1)))
+                     / priv->n_cols + 0.5);
+  row_height = (int) ((content_box->y2 - content_box->y1
+                      - (row_spacing * (priv->n_rows - 1)))
+                      / priv->n_rows + 0.5);
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       gint row, col, row_span, col_span;
       StTableChild *meta;
@@ -357,7 +300,7 @@ st_table_calculate_col_widths (StTable *table,
   gboolean *is_expand_col;
   gint extra_col_width, n_expanded_cols = 0, expanded_cols = 0;
   gint *pref_widths, *min_widths;
-  GSList *list;
+  GList *list, *children;
 
   g_array_set_size (priv->is_expand_col, 0);
   g_array_set_size (priv->is_expand_col, priv->n_cols);
@@ -371,7 +314,9 @@ st_table_calculate_col_widths (StTable *table,
   g_array_set_size (priv->min_widths, priv->n_cols);
   min_widths = (gint *) priv->min_widths->data;
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (table));
+
+  for (list = children; list; list = list->next)
     {
       gint row, col;
       gfloat w_min, w_pref;
@@ -459,7 +404,7 @@ st_table_calculate_row_heights (StTable *table,
                                 gint   * col_widths)
 {
   StTablePrivate *priv = ST_TABLE (table)->priv;
-  GSList *list;
+  GList *list, *children;
   gint *is_expand_row, *min_heights, *pref_heights, *row_heights, extra_row_height;
   gint i, total_min_height;
   gint expanded_rows = 0;
@@ -481,8 +426,8 @@ st_table_calculate_row_heights (StTable *table,
   g_array_set_size (priv->pref_heights, priv->n_rows);
   pref_heights = (gboolean *) priv->pref_heights->data;
 
-  /* calculate minimum row widths and column heights */
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (table));
+  for (list = children; list; list = list->next)
     {
       gint row, col, cell_width;
       gfloat h_min, h_pref;
@@ -633,7 +578,7 @@ st_table_preferred_allocate (ClutterActor          *self,
                              const ClutterActorBox *content_box,
                              gboolean               flags)
 {
-  GSList *list;
+  GList *list, *children;
   gint row_spacing, col_spacing;
   gint i;
   gint *col_widths, *row_heights;
@@ -658,8 +603,8 @@ st_table_preferred_allocate (ClutterActor          *self,
 
   ltr = (st_widget_get_direction (ST_WIDGET (self)) == ST_TEXT_DIRECTION_LTR);
 
-
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       gint row, col, row_span, col_span;
       gint col_width, row_height;
@@ -803,7 +748,7 @@ st_table_get_preferred_width (ClutterActor *self,
   gfloat total_min_width, total_pref_width;
   StTablePrivate *priv = ST_TABLE (self)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
-  GSList *list;
+  GList *list, *children;
   gint i;
 
   if (priv->n_cols < 1)
@@ -825,7 +770,8 @@ st_table_get_preferred_width (ClutterActor *self,
   pref_widths = (gint *) priv->pref_widths->data;
 
   /* calculate minimum row widths */
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       gint col, col_span;
       gfloat w_min, w_pref;
@@ -860,6 +806,14 @@ st_table_get_preferred_width (ClutterActor *self,
       total_pref_width += pref_widths[i];
     }
 
+  /* If we were requested width-for-height, then we reported minimum/natural
+   * heights based on our natural width. If we were allocated less than our
+   * natural width, then we need more height. So in the width-for-height
+   * case we need to disable shrinking.
+   */
+  if (for_height >= 0)
+    total_min_width = total_pref_width;
+
   if (min_width_p)
     *min_width_p = total_min_width;
   if (natural_width_p)
@@ -878,10 +832,21 @@ st_table_get_preferred_height (ClutterActor *self,
   gfloat total_min_height, total_pref_height;
   StTablePrivate *priv = ST_TABLE (self)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
-  GSList *list;
+  GList *list, *children;
   gint i;
   gint *min_widths;
 
+  /* We only support height-for-width allocation. So if we are called
+   * width-for-height, calculate heights based on our natural width
+   */
+  if (for_width < 0)
+    {
+      float natural_width;
+
+      clutter_actor_get_preferred_width (self, -1, NULL, &natural_width);
+      for_width = natural_width;
+    }
+
   if (priv->n_rows < 1)
     {
       *min_height_p = 0;
@@ -906,7 +871,8 @@ st_table_get_preferred_height (ClutterActor *self,
   pref_heights = (gint *) priv->pref_heights->data;
 
   /* calculate minimum row heights */
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       gint row, col, col_span, cell_width, row_span;
       gfloat min, pref;
@@ -960,13 +926,13 @@ st_table_get_preferred_height (ClutterActor *self,
 static void
 st_table_paint (ClutterActor *self)
 {
-  StTablePrivate *priv = ST_TABLE (self)->priv;
-  GSList *list;
+  GList *list, *children;
 
   /* make sure the background gets painted first */
   CLUTTER_ACTOR_CLASS (st_table_parent_class)->paint (self);
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       ClutterActor *child = CLUTTER_ACTOR (list->data);
       if (CLUTTER_ACTOR_IS_VISIBLE (child))
@@ -978,13 +944,13 @@ static void
 st_table_pick (ClutterActor       *self,
                const ClutterColor *color)
 {
-  StTablePrivate *priv = ST_TABLE (self)->priv;
-  GSList *list;
+  GList *list, *children;
 
   /* Chain up so we get a bounding box painted (if we are reactive) */
   CLUTTER_ACTOR_CLASS (st_table_parent_class)->pick (self, color);
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+  for (list = children; list; list = list->next)
     {
       if (CLUTTER_ACTOR_IS_VISIBLE (list->data))
         clutter_actor_paint (CLUTTER_ACTOR (list->data));
@@ -994,10 +960,10 @@ st_table_pick (ClutterActor       *self,
 static void
 st_table_show_all (ClutterActor *table)
 {
-  StTablePrivate *priv = ST_TABLE (table)->priv;
-  GSList *l;
+  GList *l, *children;
 
-  for (l = priv->children; l; l = l->next)
+  children = st_container_get_children_list (ST_CONTAINER (table));
+  for (l = children; l; l = l->next)
     clutter_actor_show_all (CLUTTER_ACTOR (l->data));
 
   clutter_actor_show (table);
@@ -1006,12 +972,12 @@ st_table_show_all (ClutterActor *table)
 static void
 st_table_hide_all (ClutterActor *table)
 {
-  StTablePrivate *priv = ST_TABLE (table)->priv;
-  GSList *l;
+  GList *l, *children;
 
   clutter_actor_hide (table);
 
-  for (l = priv->children; l; l = l->next)
+  children = st_container_get_children_list (ST_CONTAINER (table));
+  for (l = children; l; l = l->next)
     clutter_actor_hide_all (CLUTTER_ACTOR (l->data));
 }
 
@@ -1049,7 +1015,6 @@ st_table_class_init (StTableClass *klass)
 
   gobject_class->set_property = st_table_set_property;
   gobject_class->get_property = st_table_get_property;
-  gobject_class->dispose = st_table_dispose;
   gobject_class->finalize = st_table_finalize;
 
   actor_class->paint = st_table_paint;
